logmode +addin
include ".\matlab.prg"

'check a workfile exists
%wfname = @wfname
if %wfname="" then
	@uiprompt("No workfile is open")
	stop
endif

'assign default values to arguments
!result =0
!choice=1
!check = 1
!dogui=1
%eqlist=""
%coefvec="gbasscoef"
%initvec =""
%startvec=""
%smpl = @pagesmpl

'get the workfile's initial sample period
%wfsample = @pagesmpl

''command line settings
if @len(@option(1)) > 0 then
	%temp = @equaloption("eqn")
	if @len(%temp) > 0 then
		%eqlist = %temp
	endif
	%temp = @equaloption("x0")
	if @len(%temp)>0 then
		%initvec= %temp
	endif
	%temp = @equaloption("smpl")
	if @len(%temp)>0 then
		%smpl = %temp
	endif		
	%temp = @equaloption("m")
	if @len(%temp)>0 then
		!choice = @val(%temp)
	endif
	%temp = @equaloption("c0")
	if @len(%temp)>0 then
		%startvec = %temp
	endif
      !check = @hasoption("cum")
      !doGUI = @hasoption("prompt")		
endif

if !dogui = 1 then
!result=@uidialog("caption","Generalized BASS Model", _
						"edit",%eqlist,"Dependent variable followed by list of exogenous variables (but no constant)", 80, _
						"check",!check,"Use cumulated values of the dependent variable", _
						"edit", %initvec,"Vector of initial values for exogenous variables (optional)", _
						"edit", %smpl, "Sample period", _
						"Radio", !choice,"Specify starting values for coefficents","Default User MATLAB(*)", _
						"edit", %startvec,"Vector of starting values for coefficients (If User option above is selected)", _
						"text","(*)MATLAB Optimization Toolbox should be installed for this option")
'if user hit cancel, then stop
if !result=-1 then
	stop
endif
endif

'sample adjustments
call samplecheck(%smpl)

'Generate the initial values for exogenous variables
!size = @wcount(%eqlist)-1
if !size = 0 then
vector(1) init = 1
 else if !size>0 and @isempty(%initvec) = 1 then
        vector(!size) init
          for !i = 1 to !size
           %ser = @word(%eqlist,!i+1)
           !first = @dtoo(%beg)
           init(!i) = {%ser}(!first)
          next
  else if !size>0 and @rows({%initvec}) <> !size  then
      %err = "Size of the vector must equal to" + " " +@str(!size)+ "." +" " + "Default settings will be used."
	@uiprompt(%err)
        vector(!size) init
          for !i = 1 to !size
           %ser = @word(%eqlist,!i+1)
           !first = @dtoo(%beg)
           init(!i) = {%ser}(!first)
          next
     else if !size>0 and @rows({%initvec}) = !size 
      vector init = {%initvec}
   endif
  endif
 endif
endif

'Generate the random starting values for coefficients
%dep = @word(%eqlist,1)
vector(!size+3) start

if !choice=1 then
  start(1) = @rnd/20
  start(2) = @rnd/2
  start(3) = @sum({%dep})
  else if !choice = 3 then
     call fmin(%eqlist,init)
       else if !choice=2 and @isempty(%startvec)=1 then
  	   @uiprompt("Vector of starting values is not supplied. Default settings will be used.")
         start(1) = @rnd/20
         start(2) = @rnd/2
         start(3) = @sum({%dep})
           else if !choice=2 and @rows({%startvec})<>!size+3 then
            %err = "Size of the vector must equal to" + " " +@str(!size+3)+ "." +" " + "Default settings will be used."
	       @uiprompt(%err)
             start(1) = @rnd/20
             start(2) = @rnd/2
             start(3) = @sum({%dep})
               else if !choice=2 and @rows({%startvec})=!size+3 then
                start = {%startvec}
              endif
         endif
      endif
  endif
endif

'Run the model
call gbass(%eqlist,start,init,%beg,%end)

smpl {%wfsample}


'*********************************************************************************************************************************************

subroutine gbass(string %list,vector cinit,vector xvarinit,string %beg,string %end)
 
'Total number of variables
!n = @wcount(%list)

'Determine the dependent variable
if !check=1 then
  %dep = @word(%list,1)
  %cumdep = "cum_"+%dep
  series {%cumdep} = @cumsum({%dep})
else
  %dep = @word(%list,1)
endif

'Initialize the coefficients to be estimated
c(1) = cinit(1)
c(2) = cinit(2)
c(3) = cinit(3)

'Create the mapping function(s)
%map = "@cumsum(1)"
%mapd = "1"
if !n>1 then
   for !k = 1 to !n-1
    c(!k+3) = cinit(!k+3)
   next
   !j=1
   while !j<!n
    !j=!j+1
    %map = %map + "+" + "c("+@str(!j+2)+")*log("+@word(%list,!j)+"/"+@str(xvarinit(!j-1))+")"
      if !check=0 then
         %mapd = %mapd + "+" + "c("+@str(!j+2)+")*@pc("+@word(%list,!j)+")/100"
      endif
   wend
endif

'Final formula for EViews
if !check = 1 then
  %function = %cumdep +"= c(3)*(1-exp(-(" + %map + ")*(c(1)+c(2))))/((c(2)/c(1))*exp(-(" +%map + ")*(c(1)+c(2)))+1)"
else
  %function = %dep +"= c(3)*("+%mapd+")*(((c(1)+c(2))^2)/c(1))*"+"(exp(-(" + %map + ")*(c(1)+c(2))))/((c(2)/c(1))*exp(-(" +%map + ")*(c(1)+c(2)))+1)^2"
endif

'Estimate the equation (nonlinear least squares)
smpl %beg %end
%eqname = "gbasseq"
%eqname = @makevalidname(%eqname)
equation {%eqname}.ls(showopts,c=1e-06) {%function}

'Get rid of the unnecessary stuff
delete cinit xvarinit

endsub

'*********************************************************************************************************************************************

subroutine samplecheck(string %smp)
	'convert magic @all smpl keyword into a pairwise sample specification:
	if %smp = "@all" then
		%smp = "@first @last"
	endif
	
	'check the sample specification doesn't have an IF condition
	if @instr(%smp,"if")>0 then
		@uiprompt("Sample if conditions are not allowed.")
		stop
	endif
	
	'check that the entered sample specification is a pair of dates
	if @wcount(%smp)>2 then
		@uiprompt("Too many dates given in sample specification.")
		stop
	endif
	
	'check that the entered sample is a valid sample
	sample _mysmpl {%smp}
	smpl _mysmpl
	smpl @all
	d _mysmpl
	
	'create sample pairs to pass into the gbass subroutine
	%beg = @word(%smp,1)
	%end = @word(%smp,2)
endsub

'*********************************************************************************************************************************************
